package chapter5.tcp;

import java.io.*;
import java.security.*;
import java.util.Arrays;
import javax.net.ssl.*;

public class SSLCounterServer
{
    public static void main(String[] args) throws Exception
    {
        int counter = 0;

        String ksName = "oechsleprivate.keystore";
        char ksPass[] = "oechsle".toCharArray();
        char ctPass[] = "oechsle".toCharArray();
        KeyStore ks = KeyStore.getInstance("JKS");
        ks.load(new FileInputStream(ksName), ksPass);
        KeyManagerFactory kmf = KeyManagerFactory.getInstance("SunX509");
        kmf.init(ks, ctPass);
        SSLContext sc = SSLContext.getInstance("TLS");
        sc.init(kmf.getKeyManagers(), null, null);
        SSLServerSocketFactory ssf = sc.getServerSocketFactory();
        try(SSLServerSocket s = (SSLServerSocket) ssf.createServerSocket(1250))
        {
            printServerSocketInfo(s);
            while(true)
            {
                try(TCPSocket tcpSocket = new TCPSocket(s.accept()))
                {
                    while(true)
                    {
                        String request = tcpSocket.receiveLine();
                        if(request != null)
                        {
                            if(request.equals("increment"))
                            {
                                counter++;
                            }
                            else if(request.equals("reset"))
                            {
                                counter = 0;
                                System.out.println("counter has been reset");
                            }
                            String result = String.valueOf(counter);
                            tcpSocket.sendLine(result);
                        }
                        else
                        {
                            System.out.println("Schlieen der Verbindung");
                            break;
                        }
                    }
                }
                catch(Exception e)
                {
                    System.out.println(e);
                    System.out.println("=> Verbindung geschlossen");
                }
            }
        }
        catch(Exception e)
        {
            System.out.println("Fehler beim Erzeugen oder Nutzen des ServerSockets");
        }
    }

    private static void printServerSocketInfo(SSLServerSocket s)
    {
        System.out.println("Server socket class: " + s.getClass());
        System.out.println("   Socket address = " + s.getInetAddress().toString());
        System.out.println("   Socket port = " + s.getLocalPort());
        System.out.println("   Enabled protocols = " + Arrays.toString(s.getEnabledProtocols()));
        System.out.println("   Enabled cipher suites = " + Arrays.toString(s.getEnabledCipherSuites()));
        System.out.println("   Enable session creation = " + s.getEnableSessionCreation());
        System.out.println("   Need client authentication = " + s.getNeedClientAuth());
        System.out.println("   Want client authentication = " + s.getWantClientAuth());
        System.out.println("   Use client mode = " + s.getUseClientMode());
    }
}