package chapter7;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import javax.jms.*;
import javax.naming.*;

public class ReceiverBlocking implements AutoCloseable
{
    private static final int NUMBER_OF_RECEIVERS = 3;
    
    private JMSContext jmsContext;
    private JMSConsumer consumer;
    
    public ReceiverBlocking(String sourceName) throws NamingException 
    {
        InitialContext initialContext = new InitialContext();
        ConnectionFactory connectionFactory = (ConnectionFactory) initialContext.lookup("ConnectionFactory");
        Destination source = (Destination) initialContext.lookup(sourceName);
        jmsContext = connectionFactory.createContext(JMSContext.CLIENT_ACKNOWLEDGE);
        consumer = jmsContext.createConsumer(source);
    }
    
    public String receive() throws JMSException
    {
        TextMessage m = (TextMessage) consumer.receive();
        String message = m.getText();
        System.out.println("Prozess " + ProcessHandle.current().pid() +
                           " / Thread " + Thread.currentThread().getName() +
                           " hat empfangen: " + message);
        //jmsContext.acknowledge();
        return message;
    }
    
    public void close()
    {
        jmsContext.close();
    }

    public static void main(String[] args)
    {
        if(args.length != 1)
        {
            System.out.println("Argument: ReceiveDestination [Queue | Topic]");
            return;
        }
        String receiveDestination = args[0];
        for(int i = 0; i < NUMBER_OF_RECEIVERS; i++)
        {
            Thread t = new Thread(()->runReceiverLoop(receiveDestination));
            t.setDaemon(true);
            t.start();
        }

        BufferedReader input = new BufferedReader(new InputStreamReader(System.in));
        System.out.println("Drcken Sie Return zum Beenden des Empfngers:");
        System.out.print(">");
        try
        {
            input.readLine();
        }
        catch(IOException e)
        {
        }
        return;
    }
    
    private static void runReceiverLoop(String receiveDestination)
    {
        try(ReceiverBlocking receiver = new ReceiverBlocking(receiveDestination))
        {
            while(true)
            {
                String message = receiver.receive();
                if(message.equals("exitrec"))
                {
                    break;
                }
            }
        }
        catch(NamingException | JMSException e)
        {
            System.out.println("Ausnahme " + e.getClass().getName() + " in main!");
        }
    }
}