package chapter7;

import java.io.*;
import javax.jms.*;
import javax.naming.*;

public class ReceiverListening implements AutoCloseable, MessageListener
{
    private static final int NUMBER_OF_RECEIVERS = 3;
    
    private JMSContext jmsContext;
    private JMSConsumer consumer;
    
    public ReceiverListening(String sourceName) throws NamingException 
    {
        InitialContext initialContext = new InitialContext();
        ConnectionFactory connectionFactory = (ConnectionFactory) initialContext.lookup("ConnectionFactory");
        Destination source = (Destination) initialContext.lookup(sourceName);
        jmsContext = connectionFactory.createContext();
        consumer = jmsContext.createConsumer(source);
        consumer.setMessageListener(this);
    }

    @Override
    public void onMessage(Message m)
    {
        try
        {
            TextMessage message = (TextMessage)m;
            System.out.println("Prozess " + ProcessHandle.current().pid() +
                            " / Thread " + Thread.currentThread().getName() +
                            " hat empfangen: " + message.getText());
        }
        catch(Exception e)
        {
            System.out.println("Ausnahme " + e.getClass().getName() + " in onMessage!");
        }
//        if(Math.random() < 1)
//        {
//            throw new java.lang.IllegalStateException();
//        }
    }
    
    public void close()
    {
        jmsContext.close();
    }

    public static void main(String[] args)
    {
        if(args.length != 1)
        {
            System.out.println("Argument: ReceiveDestination [Queue | Topic]");
            return;
        }
        String receiveDestination = args[0];
        try
        {
            for(int i = 0; i < NUMBER_OF_RECEIVERS; i++)
            {
                new ReceiverListening(receiveDestination);
            }

            BufferedReader input = new BufferedReader(new InputStreamReader(System.in));
            System.out.println("Drcken Sie Return zum Beenden des Empfngers:");
            System.out.print(">");
            input.readLine();
            return;
        }
        catch(NamingException | IOException e)
        {
            System.out.println("Ausnahme " + e.getClass().getName() + " in main!");
        }
    }
}