package chapter7;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import javax.jms.*;
import javax.naming.*;

class Counter
{
    private int counter;

    public synchronized int increment()
    {
        counter++;
        return counter;
    }

    public synchronized int reset()
    {
        counter = 0;
        return counter;
    }

    public synchronized int getCounter()
    {
        return counter;
    }
}

public class Replier implements AutoCloseable
{
    private static final int NUMBER_OF_REPLIERS = 3;
    
    private JMSContext jmsContext;
    private JMSConsumer consumer;
    private JMSProducer producer;
    private Counter counter;
    
    public Replier(String requestQueueName, Counter counter) throws NamingException 
    {
        InitialContext initialContext = new InitialContext();
        ConnectionFactory connectionFactory = (ConnectionFactory) initialContext.lookup("ConnectionFactory");
        Queue source = (Queue) initialContext.lookup(requestQueueName);
        jmsContext = connectionFactory.createContext(JMSContext.AUTO_ACKNOWLEDGE);
        consumer = jmsContext.createConsumer(source);
        producer = jmsContext.createProducer();
        this.counter = counter;
    }
    
    public void receiveAndReply() throws JMSException
    {
        TextMessage m = (TextMessage) consumer.receive();
        String message = m.getText();
        String correlationID = m.getJMSCorrelationID();
        Queue replyQueue = (Queue) m.getJMSReplyTo();
        System.out.println(" Thread " + Thread.currentThread().getName() +
                           " hat empfangen: " + message + " [ID: " + correlationID + "]");
        
        int value = switch(message)
        {
            case "reset" -> counter.reset();
            case "increment" -> counter.increment();
            default -> counter.getCounter();
        };
        producer.setJMSCorrelationID(correlationID).send(replyQueue, "" + value);
    }
    
    public void close()
    {
        jmsContext.close();
    }

    public static void main(String[] args)
    {
        if(args.length != 1)
        {
            System.out.println("Argument: RequestQueue");
            return;
        }
        String requestQueue = args[0];
        Counter counter = new Counter();
        for(int i = 0; i < NUMBER_OF_REPLIERS; i++)
        {
            Thread t = new Thread(()->runReplierLoop(requestQueue, counter));
            t.setDaemon(true);
            t.start();
        }

        BufferedReader input = new BufferedReader(new InputStreamReader(System.in));
        System.out.println("Drcken Sie Return zum Beenden des Repliers:");
        System.out.print(">");
        try
        {
            input.readLine();
        }
        catch(IOException e)
        {
        }
    }
    
    private static void runReplierLoop(String requestQueue, Counter counter)
    {
        try(Replier replier = new Replier(requestQueue, counter))
        {
            while(true)
            {
                try
                {
                    replier.receiveAndReply();
                }
                catch(JMSException e)
                {
                    System.out.println("Ausnahme " + e.getClass().getName() + " in runReplierLoop!");
                }
            }
        }
        catch(NamingException e)
        {
            System.out.println("Ausnahme " + e.getClass().getName() + " in runReplierLoop!");
        }
    }
}