package chapter7;

import javax.jms.*;
import javax.naming.*;

public class Requester implements AutoCloseable
{
    private JMSContext jmsContext;
    private JMSProducer producer;
    private JMSConsumer consumer;
    private Queue requestQueue;
    private TemporaryQueue replyQueue;
    private int correlationNumber;
    
    public Requester(String requestQueueName) throws NamingException 
    {
        InitialContext initialContext = new InitialContext();
        ConnectionFactory connectionFactory = (ConnectionFactory) initialContext.lookup("ConnectionFactory");
        requestQueue = (Queue) initialContext.lookup(requestQueueName);
        jmsContext = connectionFactory.createContext();
        producer = jmsContext.createProducer();
        replyQueue = jmsContext.createTemporaryQueue();
        producer.setJMSReplyTo(replyQueue);
        consumer = jmsContext.createConsumer(replyQueue);
    }
    
    public String request(String message) throws Exception
    {
        TextMessage textMessage = jmsContext.createTextMessage(message);
        correlationNumber++;
        String correlationID = "" + correlationNumber;
        producer.setJMSCorrelationID(correlationID).send(requestQueue, textMessage);
        TextMessage m = (TextMessage) consumer.receive();
        if(!correlationID.equals(m.getJMSCorrelationID()))
        {
            throw new Exception("Antwort mit falscher CorrelationID empfangen: " + 
                                m.getJMSCorrelationID());
        }
        return m.getText();
    }
    
    public void close()
    {
        jmsContext.close();
    }

    public static void main(String[] args)
    {
        if(args.length != 2)
        {
            System.out.println("Argumente: RequestQueue Iterations");
            return;
        }
        String requestQueue = args[0];
        int iterations = 0;
        try
        {
            iterations = Integer.parseInt(args[1]);
        }
        catch(NumberFormatException e)
        {
            System.out.println("Zweites Argument muss int-Wert sein.");
            return;
        }
        try(Requester requester = new Requester(requestQueue))
        {
            String reply = requester.request("reset");
            System.out.println("Antwort auf reset: " + reply);
            for(int i = 0; i < iterations; i++)
            {
                reply = requester.request("increment");
                System.out.println("Antwort auf increment: " + reply);
            }
        }
        catch(Exception e)
        {
            System.out.println("Ausnahme " + e.getClass().getName() + " in main!");
        }
    }
}