package chapter7;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
//import java.util.Hashtable;
import javax.jms.*;
import javax.naming.*;

public class Sender implements AutoCloseable
{
    private JMSContext jmsContext;
    private JMSProducer producer;
    private Destination destination;
    
    public Sender(String destinationName) throws NamingException 
    {
        /*
        Hashtable<String, String> props = new Hashtable<>();
        props.put("java.naming.factory.initial", "org.apache.activemq.artemis.jndi.ActiveMQInitialContextFactory");
        props.put("java.naming.provider.url", "tcp://localhost:61616");
        props.put("queue.queue1", "queue1");
        props.put("queue.queue2", "queue2");
        props.put("topic.topic1", "topic1");
        props.put("topic.topic2", "topic2");
        InitialContext initialContext = new InitialContext(props);
        */
        InitialContext initialContext = new InitialContext();
        ConnectionFactory connectionFactory = (ConnectionFactory) initialContext.lookup("ConnectionFactory");
        destination = (Destination) initialContext.lookup(destinationName);
        jmsContext = connectionFactory.createContext();
        producer = jmsContext.createProducer();
    }
    
    public void send(String message)
    {
        TextMessage textMessage = jmsContext.createTextMessage(message);
        producer.send(destination, textMessage);
    }
    
    public void close()
    {
        jmsContext.close();
    }

    public static void main(String[] args)
    {
        if(args.length != 1)
        {
            System.out.println("Argument: SendDestination [Queue | Topic]");
            return;
        }
        String sendDestination = args[0];
        try(Sender sender = new Sender(sendDestination))
        {
            BufferedReader input = new BufferedReader(new InputStreamReader(System.in));
            System.out.println("Eingabe von 'exit' zum Beenden des Senders:");
            String line;
            while(true)
            {
                System.out.print(">");
                line = input.readLine();
                if(line.equals("exit"))
                {
                    break;
                }
                String[] messages = line.split(" ");
                for(String message: messages)
                {
                    sender.send(message);
                }
            }
        }
        catch(NamingException | IOException e)
        {
            System.out.println("Ausnahme " + e.getClass().getName() + " in main!");
        }
    }
}