package chapter9.apigateway.client.gui;

import java.util.*;
import com.google.gson.Gson;
import chapter9.apigateway.client.model.Cd;
import chapter9.apigateway.client.remote.RemoteRequest;
import chapter9.apigateway.client.remote.RemoteRequestException;
import javafx.application.Application;
import javafx.application.Platform;
import javafx.collections.FXCollections;
import javafx.scene.Scene;
import javafx.scene.control.Alert;
import javafx.stage.Stage;

public class Main extends Application
{
    public void start(Stage primaryStage)
    {
        Alert splash = new Alert(Alert.AlertType.INFORMATION);
        try
        {
            splash.setTitle("Laden der Daten aus der Cloud");
            splash.setHeaderText("    ");
            splash.setContentText("Bitte warten Sie, bis die Daten geladen sind.");
            splash.show();
            List<Cd> rawData = fill();
            View v = new View(FXCollections.observableList(rawData));
            splash.close();

            Scene scene = new Scene(v.getUI());
            primaryStage.setScene(scene);
            primaryStage.setTitle("Die besten CDs");            
            primaryStage.show();
        }
        catch(RemoteRequestException e)
        {
            splash.close();
            Alert alert = new Alert(Alert.AlertType.ERROR);
            alert.setTitle("Fehler beim initialen Laden der Daten");
            alert.setHeaderText(null);
            alert.setContentText("Initiale Daten knnen nicht aus der Cloud geladen werden! [" + e + "]");
            alert.showAndWait();
            Platform.exit();
        }
    }
    
    @SuppressWarnings("unchecked")
    public static List<Cd> fill() throws RemoteRequestException
    {
        String jsonString = RemoteRequest.get();
        
        List<Cd> result = new ArrayList<>();
        Map<String, Object> retMap = new Gson().fromJson(jsonString, new HashMap<String, Object>().getClass());
        jsonString = (String)retMap.get("result");

        //Beginn der Bereinigung des Strings:
        StringBuffer s = new StringBuffer(jsonString);
        int lastIndex = 0;
        do
        {
            int index1 = s.indexOf("{S: ", lastIndex);
            int index2 = s.indexOf("{N: ", lastIndex);
            int index;
            if(index1 == -1)
            {
                index = index2;
            }
            else if(index2 == -1)
            {
                index = index1;
            }
            else
            {
                index = Math.min(index1, index2);
            }
            if(index == -1)
            {
                break;
            }
            s.replace(index, index+4, "\"");
            lastIndex = index;
            
            index = s.indexOf(",}", lastIndex);
            if(index == -1)
            {
                break;
            }
            s.replace(index, index+2, "\"");
            lastIndex = index;
        } while(true);        
        s.deleteCharAt(s.length()-2);
        jsonString = s.toString();
        //Ende der Bereinigung
        
        retMap = new Gson().fromJson(jsonString, new HashMap<String, Object>().getClass());
        ArrayList<Map<String, Object>> list = (ArrayList<Map<String, Object>>)retMap.get("Items");
        for(Map<String, Object> m: list)
        {
            String id = (String)m.get("id");
            String artist = (String)m.get("artist");
            String title = (String)m.get("title");
            String year = (String)m.get("year");
            Cd cd = new Cd(id, artist, title, year);
            result.add(cd);
        }
        
        return result;
    }

    public static void main(String[] args)
    {
        launch(args);
    }
}
