package chapter9.apigateway.client.remote;

import java.io.*;
import java.net.*;
import java.util.HashMap;
import java.util.Map;
import javax.net.ssl.HttpsURLConnection;
import com.google.gson.Gson;
import chapter9.apigateway.client.model.Cd;

public class RemoteRequest
{
    private static final String URL_STRING = "https://7woowclzef.execute-api.eu-central-1.amazonaws.com/dev/cd";
    private static final String API_KEY_NAME = "x-api-key";
    private static final String API_KEY_VALUE = "IYmeWZEtTK4hfOjHzGQjq6dcW7nj4oQj5ElN46o4";

    private static String request(String method, String id, String data) throws RemoteRequestException
    {
        HttpsURLConnection connection = null;
        try
        {
            String urlString = URL_STRING;
            if(id != null)
            {
                urlString += "/" + id;
            }
            URL url = new URL(urlString);
            connection = (HttpsURLConnection) url.openConnection();
            connection.setRequestMethod(method);
            connection.setRequestProperty(API_KEY_NAME, API_KEY_VALUE);
            
            if(data != null)
            {
                connection.setDoOutput(true);
                connection.setRequestProperty("Content-Type", "application-json");
                OutputStreamWriter osw = new OutputStreamWriter(connection.getOutputStream());
                osw.write(data);
                osw.flush();
                osw.close();
            }
            
            //Beim ersten get (egal, ob getResponseCode oder getInputStream oder ...)
            //werden Daten gesendet
            int rc = connection.getResponseCode();
            if(rc < HttpURLConnection.HTTP_OK || rc >= HttpURLConnection.HTTP_MULT_CHOICE)
            {
                throw new RemoteRequestException(connection.getResponseMessage(), rc);
            }
            BufferedReader br = new BufferedReader(
                                    new InputStreamReader(
                                        connection.getInputStream()));
            StringBuffer result = new StringBuffer();
            String line;
            while((line = br.readLine()) != null)
            {
                result.append(line);
            }
            br.close();
            return result.toString();
        }
        catch(IOException e)
        {
            System.out.println("Ausnahme " + e + " => Handlungsbedarf");
            throw new RemoteRequestException(e.getMessage(), -1);
        }
        finally
        {
            if(connection != null)
            {
                connection.disconnect();
            }
        }
    }
    
    public static String get(String id) throws RemoteRequestException
    {
        return request("GET", id, null);
    }
    
    public static String get() throws RemoteRequestException
    {
        return request("GET", null, null);
    }
    
    public static String post(String data) throws RemoteRequestException
    {
        return request("POST", null, data);
    }
    
    public static String put(String id, String data) throws RemoteRequestException
    {
        return request("PUT", id, data);
    }
    
    public static String delete(String id) throws RemoteRequestException
    {
        return request("DELETE", id, null);
    }
    
    static void checkGetId(String id)
    {
        try
        {
            String result = get(id);
            System.out.println("Erfolg fr " + id + ": " + result);
        }
        catch(RemoteRequestException e)
        {
            System.out.println("Misserfolg fr " + id + ": " + e.getRc() + " / " + e.getMessage());
        }

    }
    
    @SuppressWarnings("unchecked")
    static void checkGetAll()
    {
        try
        {
            String result = get();
            System.out.println("Erfolg fr get(): " + result);
            Map<String, String> retMap = new Gson().fromJson(result, new HashMap<String, String>().getClass());
            result = retMap.get("result");
            System.out.println("===============");
            System.out.println(result);
        }
        catch(RemoteRequestException e)
        {
            System.out.println("Misserfolg fr get(): " + e.getRc() + " / " + e.getMessage());
        }
    }
    
    static void checkPost(Cd cd)
    {
        if(cd.getId() != null)
        {
            return;
        }
        try
        {
            String result = post(cd.toString());
            System.out.println("Erfolg fr post(): " + result);
        }
        catch(RemoteRequestException e)
        {
            System.out.println("Misserfolg fr post(): " + e.getRc() + " / " + e.getMessage());
        }
    }
    
    static void checkPut(Cd cd)
    {
        if(cd.getId() == null)
        {
            System.out.println("Keine Id");
            return;
        }
        try
        {
            String result = put(cd.getId(), cd.toJson());
            System.out.println("Erfolg fr put(): " + result);
        }
        catch(RemoteRequestException e)
        {
            System.out.println("Misserfolg fr put(): " + e.getRc() + " / " + e.getMessage());
        }
    }
    
    static void checkDelete()
    {
        try
        {
            String result = delete("1");
            System.out.println("Erfolg fr delete(): " + result);
        }
        catch(RemoteRequestException e)
        {
            System.out.println("Misserfolg fr delete(): " + e.getRc() + " / " + e.getMessage());
        }
    }
    
    public static void main(String[] args)
    {
        checkPut(new Cd("2", "Mark Knopfler", "Philadelphia", "2002"));
        checkGetAll();
    }
}
