package chapter9.polly;

import javafx.application.Application;
import javafx.application.Platform;
import javafx.geometry.Insets;
import javafx.scene.Scene;
import javafx.scene.control.*;
import javafx.scene.layout.*;
import javafx.stage.Stage;

public class Gui extends Application
{
    private TextArea textArea;
    private ComboBox<Language> language;
    private ComboBox<Gender> gender;
    private Button playButton;
    private PollyService service;
    
    public void start(Stage primaryStage)
    {
        VBox root = new VBox(10);
        root.setPadding(new Insets(10));

        textArea = new TextArea();
        textArea.setMaxHeight(Double.MAX_VALUE);
        textArea.setWrapText(true);
        VBox.setVgrow(textArea, Priority.ALWAYS);
        
        HBox commandLine = new HBox(10);
        language = new ComboBox<>();
        language.getItems().addAll(Language.values());
        language.getSelectionModel().select(0);
        gender = new ComboBox<>();
        gender.getItems().addAll(Gender.values());
        gender.getSelectionModel().select(0);
        playButton = new Button("Abspielen");
        Button clearButton = new Button("Inhalt lschen");
        commandLine.getChildren().addAll(language, gender, playButton, clearButton);
        
        root.getChildren().addAll(commandLine, textArea);

        playButton.setOnAction(e -> 
                      play(textArea.getText(), language.getSelectionModel().getSelectedItem(),
                           gender.getSelectionModel().getSelectedItem()));
        clearButton.setOnAction(e -> textArea.clear());
        
        Scene scene = new Scene(root, 1000, 250);
        primaryStage.setScene(scene);
        primaryStage.setTitle("Text zu Sprache");
        
        try
        {
            service = new PollyService();
        }
        catch(Exception e)
        {
            Alert alert = new Alert(Alert.AlertType.ERROR);
            alert.setTitle("Fehler");
            alert.setHeaderText("Dienst nicht initialisiert");
            alert.setHeaderText(null);
            alert.setContentText("Der Dienst zur Wandlung von Text in Sprache" +
                                 " ist nicht initialisiert und steht deshalb" +
                                 " nicht zur Verfgung!");
            alert.showAndWait();
            enableButton(false);
        }

        primaryStage.show();
    }
    
    private void enableButton(boolean enabled)
    {
        playButton.setDisable(!enabled);
    }

    private void play(String text, Language language, Gender gender)
    {
        if(language == null)
        {
            Alert alert = new Alert(Alert.AlertType.ERROR);
            alert.setTitle("Fehler");
            alert.setHeaderText("Sprache nicht gewhlt");
            alert.setHeaderText(null);
            alert.setContentText("Es wurde keine Sprache ausgewhlt!");
            alert.showAndWait();
            return;
        }
        if(gender == null)
        {
            Alert alert = new Alert(Alert.AlertType.ERROR);
            alert.setTitle("Fehler");
            alert.setHeaderText("Geschlecht nicht gewhlt");
            alert.setHeaderText(null);
            alert.setContentText("Es wurde kein Geschlecht ausgewhlt!");
            alert.showAndWait();
            return;
        }
        enableButton(false);
        Thread thread = new Thread(() -> playAsync(text, language.getLanguageCode(), gender.getGenderCode()));
        thread.start();
    }

    private void playAsync(String text, String languageCode, String genderCode)
    {
        try
        {
            service.speak(text, languageCode, genderCode, () -> enableButton(true));
        }
        catch(Exception e)
        {
            Platform.runLater(() -> enableButton(true));
        }
    }

    public static void main(String[] args)
    {
        launch(args);
    }
}
