package chapter9.polly;

import java.io.IOException;
import java.io.InputStream;
import java.util.*;
import com.amazonaws.ClientConfiguration;
import com.amazonaws.auth.profile.ProfileCredentialsProvider;
import com.amazonaws.regions.Region;
import com.amazonaws.regions.Regions;
import com.amazonaws.services.polly.AmazonPollyClient;
import com.amazonaws.services.polly.model.DescribeVoicesRequest;
import com.amazonaws.services.polly.model.DescribeVoicesResult;
import com.amazonaws.services.polly.model.OutputFormat;
import com.amazonaws.services.polly.model.SynthesizeSpeechRequest;
import com.amazonaws.services.polly.model.SynthesizeSpeechResult;
import com.amazonaws.services.polly.model.Voice;

import javazoom.jl.player.advanced.AdvancedPlayer;
import javazoom.jl.player.advanced.PlaybackEvent;
import javazoom.jl.player.advanced.PlaybackListener;

public class PollyDemo
{
    private final AmazonPollyClient polly;
    private final Voice voice;
    private static final String SAMPLE = "Congratulations. You have successfully built this working demo\n" +
                                         "of Amazon Polly in Java. Have fun building voice enabled\n" +
                                         "apps with Amazon Polly (that's me!), and always look at\n" +
                                         "the AWS website for tips and tricks on using Amazon Polly\n" +
                                         "and other great services from AWS";

    public PollyDemo(Region region, String language, String gender)
    {
        // create an Amazon Polly client in a specific region
        polly = new AmazonPollyClient(new ProfileCredentialsProvider("Polly"), new ClientConfiguration());
        polly.setRegion(region);
        // Create describe voices request.
        DescribeVoicesRequest describeVoicesRequest = new DescribeVoicesRequest();

        // Synchronously ask Amazon Polly to describe available TTS voices.
        DescribeVoicesResult describeVoicesResult = polly.describeVoices(describeVoicesRequest);
        List<Voice> matchingVoices = new ArrayList<>();
        List<Voice> allVoices = describeVoicesResult.getVoices();
        for(Voice v: allVoices)
        {
            if(v.getGender().equals(gender) && v.getLanguageCode().startsWith(language))
            {
                matchingVoices.add(v);
            }
        }
        if(matchingVoices.size() > 0)
        {
            voice = matchingVoices.get((int)(Math.random() * matchingVoices.size()));
        }
        else
        {

            voice = allVoices.get((int)(Math.random() * allVoices.size()));
        }
        System.out.println(matchingVoices.size());
        System.out.println(voice);
    }

    public InputStream synthesize(String text, OutputFormat format) throws IOException
    {
        SynthesizeSpeechRequest synthReq = new SynthesizeSpeechRequest().withText(text).withVoiceId(voice.getId()).withOutputFormat(format);
        SynthesizeSpeechResult synthRes = polly.synthesizeSpeech(synthReq);

        return synthRes.getAudioStream();
    }

    public static void main(String args[]) throws Exception
    {
        // create the test class
        PollyDemo helloWorld = new PollyDemo(Region.getRegion(Regions.EU_CENTRAL_1), "fr", "Female");
        // get the audio stream
        InputStream speechStream = helloWorld.synthesize(SAMPLE, OutputFormat.Mp3);

        // create an MP3 player
        AdvancedPlayer player = new AdvancedPlayer(speechStream, javazoom.jl.player.FactoryRegistry.systemRegistry().createAudioDevice());

        player.setPlayBackListener(new PlaybackListener()
        {
            @Override
            public void playbackStarted(PlaybackEvent evt)
            {
                System.out.println("Playback started");
                System.out.println(SAMPLE);
            }

            @Override
            public void playbackFinished(PlaybackEvent evt)
            {
                System.out.println("Playback finished");
            }
        });

        // play it!
        player.play();
    }
}