package chapter9.polly;

import java.util.ArrayList;
import java.util.List;
import com.amazonaws.ClientConfiguration;
import com.amazonaws.auth.profile.ProfileCredentialsProvider;
import com.amazonaws.regions.Region;
import com.amazonaws.regions.Regions;
import com.amazonaws.services.polly.AmazonPollyClient;
import com.amazonaws.services.polly.model.DescribeVoicesRequest;
import com.amazonaws.services.polly.model.DescribeVoicesResult;
import com.amazonaws.services.polly.model.OutputFormat;
import com.amazonaws.services.polly.model.SynthesizeSpeechRequest;
import com.amazonaws.services.polly.model.SynthesizeSpeechResult;
import com.amazonaws.services.polly.model.Voice;
import javafx.application.Platform;
import javazoom.jl.decoder.JavaLayerException;
import javazoom.jl.player.advanced.AdvancedPlayer;
import javazoom.jl.player.advanced.PlaybackEvent;
import javazoom.jl.player.advanced.PlaybackListener;

public class PollyService
{
    private final AmazonPollyClient polly;
    private final List<Voice> allVoices;
    
    public PollyService()
    {
        // create an Amazon Polly client in a specific region
        polly = new AmazonPollyClient(new ProfileCredentialsProvider("Polly"), new ClientConfiguration());
        polly.setRegion(Region.getRegion(Regions.EU_CENTRAL_1));
        // Create describe voices request.
        DescribeVoicesRequest describeVoicesRequest = new DescribeVoicesRequest();

        // Synchronously ask Amazon Polly to describe available TTS voices.
        DescribeVoicesResult describeVoicesResult = polly.describeVoices(describeVoicesRequest);
        allVoices = describeVoicesResult.getVoices();
    }
    
    public void speak(String text, String language, String gender, Runnable finalActivity) throws JavaLayerException
    {
        List<Voice> matchingVoices = new ArrayList<>();
        Voice voice;
        for(Voice v: allVoices)
        {
            if(v.getGender().equals(gender) && v.getLanguageCode().startsWith(language))
            {
                matchingVoices.add(v);
            }
        }
        if(matchingVoices.size() > 0)
        {
            voice = matchingVoices.get((int)(Math.random() * matchingVoices.size()));
        }
        else
        {

            voice = allVoices.get((int)(Math.random() * allVoices.size()));
        }
        SynthesizeSpeechRequest synthReq = new SynthesizeSpeechRequest().withText(text).withVoiceId(voice.getId()).withOutputFormat(OutputFormat.Mp3);
        SynthesizeSpeechResult synthRes = polly.synthesizeSpeech(synthReq);
        // create an MP3 player
        AdvancedPlayer player = new AdvancedPlayer(synthRes.getAudioStream(), javazoom.jl.player.FactoryRegistry.systemRegistry().createAudioDevice());

        player.setPlayBackListener(new PlaybackListener()
        {
            @Override
            public void playbackStarted(PlaybackEvent evt)
            {
            }

            @Override
            public void playbackFinished(PlaybackEvent evt)
            {
                Platform.runLater(finalActivity);
            }
        });

        // play it!
        player.play();

    }
}