package chapter9.transcribe;

import java.io.*;
import java.net.URISyntaxException;
import java.util.*;
import java.util.concurrent.*;
import javax.sound.sampled.*;
import software.amazon.awssdk.auth.credentials.*;
import software.amazon.awssdk.regions.Region;
import software.amazon.awssdk.services.transcribe.model.LanguageCode;
import software.amazon.awssdk.services.transcribestreaming.TranscribeStreamingAsyncClient;
import software.amazon.awssdk.services.transcribestreaming.model.*;

public class TranscribeStreamingDemoApp
{
    private static final Region REGION = Region.EU_CENTRAL_1;
    private static TranscribeStreamingAsyncClient client;

    public static void main(String args[]) throws URISyntaxException, ExecutionException, InterruptedException, LineUnavailableException
    {

        client = TranscribeStreamingAsyncClient.builder().credentialsProvider(getCredentials()).region(REGION).build();

        InputStream input = getStreamFromMic();
        CompletableFuture<Void> result = client.startStreamTranscription(getRequest(16_000), new AudioStreamPublisher(input), getResponseHandler());

        System.out.println("Zum Beenden Eingabetaste drcken!");
        try(Scanner scanner = new Scanner(System.in))
        {
            scanner.nextLine();
            input.close();
        }
        catch(IOException e)
        {
        }

        result.get();
        client.close();
    }

    private static AwsCredentialsProvider getCredentials()
    {
        return ProfileCredentialsProvider.create("Transcribe");
    }

    private static StartStreamTranscriptionRequest getRequest(Integer mediaSampleRateHertz)
    {
        return StartStreamTranscriptionRequest.builder().languageCode(LanguageCode.DE_DE.toString()).mediaEncoding(MediaEncoding.PCM).mediaSampleRateHertz(mediaSampleRateHertz).build();
    }

    private static StartStreamTranscriptionResponseHandler getResponseHandler()
    {
        return StartStreamTranscriptionResponseHandler.builder().onResponse(r -> {
            System.out.println("Received Initial response");
        }).onError(e -> {
            System.out.println(e.getMessage());
            StringWriter sw = new StringWriter();
            e.printStackTrace(new PrintWriter(sw));
            System.out.println("Error Occurred: " + sw.toString());
        }).onComplete(() -> {
            System.out.println("=== All records stream successfully ===");
        }).subscriber(event -> {
            List<Result> results = ((TranscriptEvent) event).transcript().results();
            for(int i = 0; i < results.size(); i++)
            {
                for(int j = 0; j < results.get(i).alternatives().size(); j++)
                {
                    if(!results.get(i).alternatives().get(j).transcript().isEmpty())
                    {
                        String marker = results.get(i).isPartial()? "...": ">>>>";
                        System.out.println(marker + "(" + i + ", " + j + "): " + results.get(i).alternatives().get(j).transcript());
                    }
                }
            }
        }).build();
    }

    private static InputStream getStreamFromMic() throws LineUnavailableException
    {

        // Signed PCM AudioFormat with 16kHz, 16 bit sample size, mono
        int sampleRate = 16000;
        AudioFormat format = new AudioFormat(sampleRate, 16, 1, true, false);
        DataLine.Info info = new DataLine.Info(TargetDataLine.class, format);

        if(!AudioSystem.isLineSupported(info))
        {
            System.out.println("Line not supported");
            System.exit(0);
        }

        TargetDataLine line = (TargetDataLine) AudioSystem.getLine(info);
        line.open(format);
        line.start();

        InputStream audioStream = new AudioInputStream(line);
        return audioStream;
    }

    @SuppressWarnings("unused")
    private InputStream getStreamFromFile(String audioFileName)
    {
        try
        {
            File inputFile = new File(getClass().getClassLoader().getResource(audioFileName).getFile());
            InputStream audioStream = new FileInputStream(inputFile);
            return audioStream;
        }
        catch(FileNotFoundException e)
        {
            throw new RuntimeException(e);
        }
    }
}