package chapter9.translate;

import javafx.application.Application;
import javafx.application.Platform;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.Scene;
import javafx.scene.control.*;
import javafx.scene.layout.*;
import javafx.stage.Stage;

public class Gui extends Application
{
    private TextArea leftArea, rightArea;
    private ComboBox<Language> leftSelection, rightSelection;
    private Button l2r, r2l;
    private TranslationService service;
    
    public void start(Stage primaryStage)
    {
        HBox root = new HBox(10);
        root.setPadding(new Insets(10));
        VBox leftBox = new VBox(10);
        HBox leftCommandLine = new HBox(10);
        leftSelection = new ComboBox<>();
        leftSelection.getItems().addAll(Language.values());
        leftSelection.getSelectionModel().select(0);
        Button leftClearButton = new Button("Inhalt lschen");
        leftCommandLine.getChildren().addAll(leftSelection, leftClearButton);
        leftArea = new TextArea();
        leftArea.setMaxHeight(Double.MAX_VALUE);
        leftArea.setWrapText(true);
        VBox.setVgrow(leftArea, Priority.ALWAYS);
        leftBox.getChildren().addAll(leftCommandLine, leftArea);
        VBox middleBox = new VBox(10);
        l2r = new Button("=>");
        r2l = new Button("<=");
        middleBox.setAlignment(Pos.CENTER);
        middleBox.getChildren().addAll(l2r, r2l);
        middleBox.setMinWidth(50);
        HBox.setHgrow(middleBox, Priority.SOMETIMES);
        VBox rightBox = new VBox(10);
        HBox rightCommandLine = new HBox(10);
        rightSelection = new ComboBox<>();
        rightSelection.getItems().addAll(Language.values());
        rightSelection.getSelectionModel().select(1);
        Button rightClearButton = new Button("Inhalt lschen");
        rightCommandLine.getChildren().addAll(rightSelection, rightClearButton);
        rightArea = new TextArea();
        rightArea.setMaxHeight(Double.MAX_VALUE);
        rightArea.setWrapText(true);
        VBox.setVgrow(rightArea, Priority.ALWAYS);
        rightBox.getChildren().addAll(rightCommandLine, rightArea);
        
        root.getChildren().addAll(leftBox, middleBox, rightBox);

        leftClearButton.setOnAction(e -> leftArea.clear());
        rightClearButton.setOnAction(e -> rightArea.clear());
        
        l2r.setOnAction(e -> 
            doTranslation(leftSelection.getSelectionModel().getSelectedItem(),
                          rightSelection.getSelectionModel().getSelectedItem(),
                          leftArea.getText(), rightArea));
        r2l.setOnAction(e -> 
            doTranslation(rightSelection.getSelectionModel().getSelectedItem(),
                          leftSelection.getSelectionModel().getSelectedItem(),
                          rightArea.getText(), leftArea));
        
        try
        {
            service = new TranslationService();
        }
        catch(Exception e)
        {
            Alert alert = new Alert(Alert.AlertType.ERROR);
            alert.setTitle("Fehler");
            alert.setHeaderText("bersetzungsdienst nicht initialisiert");
            alert.setHeaderText(null);
            alert.setContentText("Der bersetzungsdienst ist nicht initialisiert und steht deshalb nicht zur Verfgung!");
            alert.showAndWait();
            Platform.exit();
            return;
        }
        
        Scene scene = new Scene(root, 1000, 250);
        primaryStage.setScene(scene);
        primaryStage.setTitle("bersetzung");
        primaryStage.show();
    }
    
    private void enableButtons(boolean enabled)
    {
        l2r.setDisable(!enabled);
        r2l.setDisable(!enabled);
    }

    private void doTranslation(Language fromLanguage, Language toLanguage,
                               String text, TextArea resultArea)
    {
        if(fromLanguage == null)
        {
            Alert alert = new Alert(Alert.AlertType.ERROR);
            alert.setTitle("Fehler");
            alert.setHeaderText("Quellsprache nicht gewhlt");
            alert.setHeaderText(null);
            alert.setContentText("Es wurde keine Quellsprache ausgewhlt!");
            alert.showAndWait();
            return;
        }
        if(toLanguage == null)
        {
            Alert alert = new Alert(Alert.AlertType.ERROR);
            alert.setTitle("Fehler");
            alert.setHeaderText("Zielsprache nicht gewhlt");
            alert.setHeaderText(null);
            alert.setContentText("Es wurde keine Zielsprache ausgewhlt!");
            alert.showAndWait();
            return;
        }
        if(fromLanguage == toLanguage)
        {
            resultArea.setText(text);
            return;
        }
        enableButtons(false);
        Thread thread = new Thread(() -> translateAsync(text, fromLanguage.getLanguageCode(), toLanguage.getLanguageCode(), resultArea));
        thread.start();
    }

    private void translateAsync(String text, String fromLanguageCode, String toLanguageCode, TextArea resultArea)
    {
        String translatedText = service.translate(text, fromLanguageCode, toLanguageCode);
        Platform.runLater(() -> resultArea.setText(translatedText));
        Platform.runLater(() -> enableButtons(true));
    }

    public static void main(String[] args)
    {
        launch(args);
    }
}
