package chapter9.translate;

import com.amazonaws.AmazonClientException;
import com.amazonaws.auth.*;
import com.amazonaws.auth.profile.ProfileCredentialsProvider;
import com.amazonaws.regions.Regions;
import com.amazonaws.services.translate.*;
import com.amazonaws.services.translate.model.*;

public class TranslationService
{
    private AmazonTranslate translate;
    
    public TranslationService()
    {
        AWSCredentials credentials = null;
        try
        {
            credentials = new ProfileCredentialsProvider("Translate").getCredentials();
        }
        catch(Exception e)
        {
            throw new AmazonClientException("Cannot load the credentials from the credential profiles file. " + "Please make sure that your credentials file is at the correct " + "location (C:\\Users\\oechsle\\.aws\\credentials), and is in valid format.", e);
        }

        translate = AmazonTranslateClient.builder().withCredentials(new AWSStaticCredentialsProvider(credentials)).withRegion(Regions.EU_CENTRAL_1).build();
    }
    
    public String translate(String text, String fromLanguage, String toLanguage)
    {
        TranslateTextRequest request = new TranslateTextRequest().withText(text).withSourceLanguageCode(fromLanguage).withTargetLanguageCode(toLanguage);
        TranslateTextResult result = translate.translateText(request);
        return result.getTranslatedText();
    }

    public static void main(String[] args)
    {
        TranslationService service = new TranslationService();
        System.out.println(service.translate("Hallo Welt", "de", "fr"));
        System.out.println(service.translate("Hallo Welt", "de", "it"));
        System.out.println(service.translate("I could drink a case of you, and I would still be on my feet", "en", "de"));
    }
}